#ifndef __BSP_H
#define __BSP_H

#include "SDL_opengl.h"
#include "Material.h"

class CBSP
{
	public:
		CBSP();
		~CBSP();

		void Setup();
		void Add(GLuint *elements, int NumElements, CMaterial *mat);
		void Compile(GLfloat *Verts);

		inline bool Empty() {return (GHead ? false : true) && (Head ? false : true);}

		void Draw();

	private:
		class CBSPNode
		{
			public:
				GLuint Elements[3];
				GLfloat Normal[3], Distance;
				CBSPNode *Back, *Front, *Undivided;
				CMaterial *Material;

				CBSPNode();
				~CBSPNode();
				void operator =(CBSPNode &rhs)
				{
					Elements[0] = rhs.Elements[0]; Elements[1] = rhs.Elements[1]; Elements[2] = rhs.Elements[2];
					Normal[0] = rhs.Normal[0]; Normal[1] = rhs.Normal[1]; Normal[2] = rhs.Normal[2];
					Distance = rhs.Distance;
					Material = rhs.Material;
				};
				void Draw(GLfloat *Matrix);

				void ComputeNormal(CBSP *Parent);
				CBSPNode *GetDivider(CBSP *Parent);

				int GetNumDivisions(CBSP *Parent, CBSPNode *Children);
				void Divide(CBSP *Parent, CBSPNode *Children);

				void Split(CBSP *Parent, CBSPNode *Splitee);
				void GetSplitPoints(CBSP *Parent, CBSPNode *Splitee, float Side);
				int GetPoint(CBSP *Parent, float *First, float *Second, float ratio);

				int ClassifyFace(CBSP *Parent, CBSPNode *Face);
		} *Head;
		friend class CBSPNode;
		
		GLfloat *Verts;
		int NumAllocdVerts;
		int CurrentVert;

		int AddVert(GLfloat *input);

		struct GeometryHolder
		{
			GLuint *Elements;
			int NumElements;
			CMaterial *Material;
			GeometryHolder *Next;
		} *GHead;
		void DeleteGeometryHolders();
};

#endif