#include "GLFoundation.h"
#include "SystemDriver.h"
#include <string.h>

char *ExtString;
void SeedExtensionSearch()
{
	ExtString = (char *)glGetString(GL_EXTENSIONS);
}

bool ExtensionSearch(char *extName)
{
	char *p = ExtString;
	char *end;
	int extNameLen;

	extNameLen = strlen(extName);
	end = p + strlen(p);

	while (p < end)
	{
		int n = strcspn(p, " ");
		if ((extNameLen == n) && (strncmp(extName, p, n) == 0))
			return GL_TRUE;
		p += (n + 1);
	}

	return GL_FALSE;
}

CGLFoundation GLFoundation;
int GLCapabilities;

#define HFOV 0.57735026919f

void CGLFoundation::SetMode(int w, int h, bool fullscreen)
{
	SystemDriver->SetMode(w, h, FullScreen = fullscreen);
	w = SystemDriver->NewWidth(); h = SystemDriver->NewHeight();
	glViewport(0, 0, w, h);

	/* enable depth testing & face culling */
	glEnable(GL_DEPTH_TEST);
	glClearDepth(1.0f);
	glDepthFunc(GL_LEQUAL);

	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	glFrontFace(GL_CCW);

	/* enable blending with the ordinary function*/
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	
	/* enable alpha testing to discard depth writes for low opaque pixels */
	glAlphaFunc(GL_GEQUAL, 8.0f / 256.0f);
	glEnable(GL_ALPHA_TEST);

	/* hint for nicest perspective correction */
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	/* set default 60 degree horizontal FOV projection matrix */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	float bound = (float)w / (float)h;
	glFrustum(-bound*HFOV, bound*HFOV, -HFOV, HFOV, 1, 2000);
	glScalef( 1.0f, 1.0f, -1.0f);

	/* load identity to modelview */
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	/* get extensions */
		GLCapabilities = 0;
		SeedExtensionSearch();

		/* compressed textures */
		if(ExtensionSearch("GL_ARB_texture_compression"))
		{
			GLCapabilities |= GLCAP_COMPRESSED_TEXTURE;
			glHint(GL_TEXTURE_COMPRESSION_HINT, GL_NICEST);
		}
}

void CGLFoundation::Flip()
{
	SystemDriver->Flip();
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}
