#include "SDL_opengl.h"
#include "Material.h"
#include "GLFoundation.h"
#include <string.h>
#include "SDL/SDL_image.h"

/*class CTexImage: public CPString
{
	public:
		bool Open(char *name);
		void Select();
	private:
		GLuint TexNum;
};

CCollection TexCollection;*/

CCollection MaterialCollection;
CMaterial *CMaterial::CurrentMaterial = NULL;

void CMaterial::Backup()
{
	if(glIsTexture(TexEnv))
	{
		delete[] BackupStore; BackupStore = NULL;

		if(!GLCapabilities&GLCAP_COMPRESSED_TEXTURE)
		{
			BackupStore = new Uint8[Height*Width*(FourChannel ? 4: 3)];
			glBindTexture(GL_TEXTURE_2D, TexEnv);
			glGetTexImage(GL_TEXTURE_2D, 0, FourChannel ? GL_RGBA : GL_RGB, GL_UNSIGNED_BYTE, BackupStore);
		}

		glDeleteTextures(1, &TexEnv);
	}
}

Uint32 mySDL_GetColour(SDL_Surface *Map, int x, int y)
{
	Uint32 SrcColour;
	Uint8 *LineBase = &((Uint8 *)Map->pixels)[(Map->h-1-y)*Map->pitch];
	switch(Map->format->BytesPerPixel)
	{
		default: SrcColour = 0; break;
		case 1: SrcColour = LineBase[x]; break;
		case 2: SrcColour = (LineBase[(x*3)] << 8) | (LineBase[(x*3)+1] << 0); break;
		case 3: SrcColour = (LineBase[(x*3)] << 16) | (LineBase[(x*3)+1] << 8) | (LineBase[(x*3)+2] << 0); break;
	}
	return SrcColour;
}

void Build2DMipmaps(GLint channels, GLint SrcWidth, GLint SrcHeight, Uint8 *Data)
{
	GLenum Format = 
		(GLCapabilities&GLCAP_COMPRESSED_TEXTURE) ? 
			((channels == 4) ? GL_COMPRESSED_RGBA_ARB : GL_COMPRESSED_RGB_ARB) : 
			((channels == 4) ? GL_RGBA : GL_RGB);

	/* upload level 0 mipmap */
	glTexImage2D(GL_TEXTURE_2D, 0, Format, SrcWidth, SrcHeight, 0, (channels == 4) ? GL_RGBA : GL_RGB, GL_UNSIGNED_BYTE, Data);

	Uint8 *DataSrc = new Uint8[SrcWidth*SrcHeight*channels];
	Uint8 *DataTarg = new Uint8[SrcWidth*SrcHeight*channels];
	memcpy(DataSrc, Data, SrcWidth*SrcHeight*channels);

	int Level = 0;
	int Pitch = SrcWidth*channels;

	while(SrcWidth > 1 || SrcHeight > 1)
	{
		SrcWidth >>= 1; if(!SrcWidth) SrcWidth = 1;
		SrcHeight >>= 1; if(!SrcHeight) SrcHeight = 1;
		Level++;

		/* idiotic box sample */
		for(int y = 0; y < SrcHeight; y++)
			for(int x = 0; x < SrcWidth; x++)
			{
				int basec = y*2*Pitch + x*2*channels;

				Uint8 r;
				int c = channels;
				while(c--)
				{
					r = (DataSrc[basec+c] + DataSrc[basec+channels+c] + DataSrc[basec+Pitch+c] + DataSrc[basec+Pitch+channels+c]) >> 2;
					DataTarg[(y*SrcWidth + x)*channels + c] = r;
				}				
			}

		glTexImage2D(GL_TEXTURE_2D, Level, Format, SrcWidth, SrcHeight, 0, (channels == 4) ? GL_RGBA : GL_RGB, GL_UNSIGNED_BYTE, DataTarg);
		
		Pitch = SrcWidth*channels;
		Uint8 *T = DataTarg; DataTarg = DataSrc; DataSrc = T;
	}

	delete[] DataTarg;
	delete[] DataSrc;
}

void CMaterial::RestoreFromDisk()
{
	SDL_Surface *Map;
	SDL_Surface *Opacity;

	Map = IMG_Load(MapName);
	if(!Map) return;
	Opacity = IMG_Load(OpacityName);

	SDL_LockSurface(Map);
	if(Opacity) SDL_LockSurface(Opacity);

	if(Opacity && Map)
	{
		if(Map->w != Opacity->w || Map->h != Opacity->h)
		{
			SDL_FreeSurface(Opacity);
			Opacity = NULL;
		}
		else
		{
			/* check whether the opacity map is just all 100% opaque */
			bool Opaque = true;
			for(int y = 0; y < Opacity->h; y++)
			{
				for(int x = 0; x < Opacity->w; x++)
				{
					Uint8 r, g, b, a;
					SDL_GetRGB(mySDL_GetColour(Opacity, x, y), Opacity->format, &r, &g, &b);
					a = (77*r + 151*g + 28*b) >> 8;
					
					if(a != 255)
					{
						Opaque = false;
						break;
					}
				}
			}

			if(Opaque)
			{
				SDL_FreeSurface(Opacity);
				Opacity = NULL;
			}
		}
	}

//	printf("%s...\n", basemat->texture1_map.name);

	if(Map)
	{
		Width = Map->w;
		Height = Map->h;

		/* package */
		CompressedBackup = false;
		delete[] BackupStore;
		if(Opacity)
		{
			FourChannel = true;

			BackupStore = new Uint8[Map->w*Map->h*4];
			for(int y = 0; y < Map->h; y++)
			{
				for(int x = 0; x < Map->w; x++)
				{
					int basec = (y*4*Map->w) + x*4;

					/* order is r, g, b, a */
					Uint8 r, g, b, a;
					SDL_GetRGB(mySDL_GetColour(Opacity, x, y), Opacity->format, &r, &g, &b);
					a = (77*r + 151*g + 28*b) >> 8;

					SDL_GetRGB(mySDL_GetColour(Map, x, y), Map->format, &r, &g, &b);

					BackupStore[basec+0] = r;
					BackupStore[basec+1] = g;
					BackupStore[basec+2] = b;
					BackupStore[basec+3] = a;
				}
			}
		}
		else
		{
			FourChannel = false;

			BackupStore = new Uint8[Map->w*Map->h*3];
			for(int y = 0; y < Map->h; y++)
			{
				for(int x = 0; x < Map->w; x++)
				{
					int basec = (y*3*Map->w) + x*3;

					/* order is r, g, b */
					Uint8 r, g, b;
					SDL_GetRGB(mySDL_GetColour(Map, x, y), Map->format, &r, &g, &b);

					BackupStore[basec+0] = r;
					BackupStore[basec+1] = g;
					BackupStore[basec+2] = b;
				}
			}
		}

		SDL_UnlockSurface(Map);
		if(Opacity) SDL_UnlockSurface(Opacity);

		/* generate texture environment, etc */
		Restore();
	}
	else
		Width = Height = 0;

	if(Opacity) SDL_FreeSurface(Opacity);
	if(Map) SDL_FreeSurface(Map);
}

void CMaterial::Restore()
{
	glGenTextures(1, &TexEnv);
	glBindTexture(GL_TEXTURE_2D, TexEnv);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

	if(BackupStore)
	{
		Build2DMipmaps(FourChannel ? 4 : 3, Width, Height, BackupStore);
		delete[] BackupStore; BackupStore = NULL;
	}
	else
		RestoreFromDisk();
}

CMaterial::CMaterial()
{
	TexEnv = 0;
	BackupStore = NULL;
}

CMaterial::~CMaterial()
{
	if(glIsTexture(TexEnv))
		glDeleteTextures(1, &TexEnv);
}

void CMaterial::Select()
{
	if(CurrentMaterial != this)
	{
		CurrentMaterial = this;

		if(glIsTexture(TexEnv))
		{
			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D, TexEnv);
		}
		else
			glDisable(GL_TEXTURE_2D);

		if(TwoSided)
			glDisable(GL_CULL_FACE);
		else
			glEnable(GL_CULL_FACE);

		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, AmbientColour);
		glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, DiffuseColour);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, SpecularColour);
		glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, Shininess);
		glColor4f(1.0f, 1.0f, 1.0f, Transparency);
	}
}

bool CMaterial::Setup(CPString &Path, Lib3dsMaterial *basemat)
{
	/* set material name */
	*dynamic_cast<CPString *>(this) = GetMaterialName(basemat);

	/* store various colours */
	int c = 4;
	while(c--)
	{
		AmbientColour[c] = basemat->ambient[c];
		DiffuseColour[c] = basemat->diffuse[c];
		SpecularColour[c] = basemat->specular[c];
	}
	Shininess = basemat->shininess;

	/* copy other flags of interest */
	TwoSided = basemat->two_sided ? true : false;
	Transparency = (100.0f - basemat->transparency) / 100.0f;

	/* load texture... */
	Path.CompletePath();
//	printf("%s %s\n", basemat->texture1_map.name, basemat->opacity_map.name);
	MapName = Path + basemat->texture1_map.name;
	OpacityName = Path + basemat->opacity_map.name;
//	printf("%s\n%s\n", (const char *)MapName, (const char *)OpacityName);
	RestoreFromDisk();

	return true;
}

CPString &GetMaterialName(Lib3dsMaterial *basemat)
{
	static CPString S;
	S = basemat->texture1_map.name;
	S += "/";
	S += basemat->opacity_map.name;
	char TempBuf[80]; // 66 should do
	sprintf(TempBuf, "/%0.2f %0.2f %0.2f %0.2f/%0.2f %0.2f %0.2f %0.2f/%0.2f %0.2f %0.2f %0.2f/%0.2f",
		basemat->ambient[0], basemat->ambient[1], basemat->ambient[2], basemat->ambient[3],
		basemat->diffuse[0], basemat->diffuse[1], basemat->diffuse[2], basemat->diffuse[3],
		basemat->specular[0], basemat->specular[1], basemat->specular[2], basemat->specular[3],
		basemat->shininess);
	S += TempBuf;
	return S;
}
