#include "SystemDriver.h"

CSystemDriver *SystemDriver;

CSystemDriver::CSystemDriver()
{
	SDL_Init(SDL_INIT_VIDEO);
	Quit = Resize = false;
}

CSystemDriver::~CSystemDriver()
{
	SDL_Quit();
}

bool CSystemDriver::SetMode(int w, int h, bool fullscreen)
{
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

	if(fullscreen)
	{
		/* adjust w and h up to next available mode or set to top mode if larger than that */
		SDL_Rect **List = SDL_ListModes(NULL, SDL_FULLSCREEN);
		if(List)
		{
			int i = 0;
			while(List[i]) i++;
			while(i--)
			{
				if(List[i]->w >= w && List[i]->h >= h)
				{
					w = List[i]->w; h = List[i]->h;
					break;
				}
			}

			if(i < 0)
			{
				w = List[0]->w; h = List[0]->h;
			}
		}
	}

	Screen = SDL_SetVideoMode(NewW = w, NewH = h, 0, SDL_OPENGL | (fullscreen ? SDL_FULLSCREEN : SDL_RESIZABLE));
	Resize = false;
	NewW = w;
	NewH = h;

	return Screen ? true : false;
}

const Uint8 *CSystemDriver::GetKeyState()
{
	int whatever;
	return SDL_GetKeyState(&whatever);
}

int CSystemDriver::GetNumKeys()
{
	int numkeys;
	SDL_GetKeyState(&numkeys);
	return numkeys;
}

void CSystemDriver::Flip()
{
	// do flip
	SDL_GL_SwapBuffers();

	// process events
	SDL_Event NewEvent;
	SDL_PumpEvents();
	while(SDL_PollEvent(&NewEvent))
	{
		switch(NewEvent.type)
		{
			case SDL_QUIT: Quit = true; break;
			case SDL_VIDEORESIZE:
				Resize = true;
				NewW = NewEvent.resize.w;
				NewH = NewEvent.resize.h;
			break;

			default: break;
		}
	}
}
